@@ -1,7 +1,9 @@
 """Agents Assembly translator"""
 
-__version__ = "0.0.58"
+__version__ = "0.1.4"
 
 from aasm.generating.code import Code
+from aasm.modules.module import Module
 from aasm.generating.python_spade import get_spade_code
+from aasm.generating.python_module import get_modules_for_target
 from aasm.utils.exception import PanicException
