@@ -2,7 +2,12 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from aasm.intermediate.graph import MatrixGraph, StatisticalGraph
+from aasm.intermediate.graph import (
+    MatrixGraph,
+    StatisticalGraph,
+    BarabasiGraph,
+    InhomogenousRandomGraph,
+)
 
 if TYPE_CHECKING:
     from aasm.parsing.state import State
@@ -33,6 +38,10 @@ def op_GRAPH(state: State, category: str) -> None:
             state.add_graph(StatisticalGraph())
         case "matrix":
             state.add_graph(MatrixGraph())
+        case "barabasi-albert":
+            state.add_graph(BarabasiGraph())
+        case "irg":
+            state.add_graph(InhomogenousRandomGraph())
         case _:
             state.panic(f"Incorrect operation: GRAPH {category}")
 
@@ -45,12 +54,24 @@ def op_EGRAPH(state: State) -> None:
     )
     if (
         isinstance(state.last_graph, StatisticalGraph)
-        and state.last_graph.is_agent_percent_amount_used()
-    ):
+        or isinstance(state.last_graph, InhomogenousRandomGraph)
+    ) and state.last_graph.is_agent_percent_amount_used():
         state.require(
             state.last_graph.is_size_defined(),
             "Graph size is not defined.",
             "Graph size must be defined to use agent percent amount.",
         )
+    elif isinstance(state.last_graph, BarabasiGraph):
+        state.require(
+            state.last_graph.is_m_defined(),
+            "Graph MParameters is not defined.",
+            "Graph MParameters must be defined to use barabasi graph.",
+        )
+        if state.last_graph.is_agent_percent_amount_used():
+            state.require(
+                state.last_graph.is_size_defined(),
+                "Graph size is not defined.",
+                "Graph size must be defined to use agent percent amount.",
+            )
 
     state.in_graph = False
